
' This is the PAK delay routine
' For a 10Mhz PAK it should delay at least 10uS
' For a 20Mhz PAK a 5uS delay is OK
' Longer delays are OK although over 2.3 seconds
' will cause some PAKs to reset
' The code uses R24 and R25 but pushes/pops them in each case
 $asm

 .def Pakctr = R25
 .def Pakdata = R24
 ; .equ Pakdpin = 4
 ; .equ Pakcpin = 3


  ; This Routine Should Delay At Least 10us For A 10mhz Pak
  ; For A 1mhz Avr You Could Probably Just Do Some Noops In Line
  ; But For Faster Chips You Will Want To Adjust The 1 Below
Pakw:
 push Pakctr
 ldi Pakctr,1  ; change this value to alter timing
Pakw0:
 DEC Pakctr
 BRNE Pakw0
 pop Pakctr
 ret
 $end Asm



' Write a byte to the PAK outputs
Sub Pakwio(byval B as Byte)
Call Paksend(&H16)
Call Paksend(b)
End Sub


' Set direction register (1=output)
Sub Pakdir(byval b as Byte)
Call Paksend(&H14)
Call Paksend(b)
End Sub

' Swap X and Y PAK registers
Sub PakSwap()
Paksend 4
End Sub

' X=X*Y
Sub PakMult()
Call Paksend(&Hc)
Pakrv = Pakrcv(1)
End Sub

' X=X+Y
Sub Pakadd()
Call Paksend(&Hf)
Pakrv = Pakrcv(1)
End Sub

' X=X-Y
Sub PakSub()
Call Paksend(&He)
Pakrv = Pakrcv(1)
End Sub

' X=X/Y
Sub PakDiv()
Call Paksend(&Hd)
Pakrv = Pakrcv(1)
End Sub


' Load 32-bit value into X
Sub Pakloadx(byval Xhi as Integer, byval Xlo as Integer)
Local T As Byte
Call Paksend(1)
T = High(xhi)
Call Paksend(t)
T = Low(xhi)
Call Paksend(t)
T = High(xlo)
Call Paksend(t)
T = Low(xlo)
Call Paksend(t)
End Function

' Load 32-bit value into Y
Sub Pakloady(byval Xhi as Integer, byval Xlo as Integer)
Local T As Byte
Call Paksend(2)
T = High(xhi)
Call Paksend(t)
T = Low(xhi)
Call Paksend(t)
T = High(xlo)
Call Paksend(t)
T = Low(xlo)
Call Paksend(t)
End Function

' Load 16-bit integer into X
' Note: PAK uses 24 bit integers but we
' just 0 fill here
Sub Pakloadint(byval X as Integer)
Call Pakloadx(0 , X)
Call Paksend(7)
Pakrv = Pakrcv(1)
End Sub

' Read X as a 16-bit integer
' Discards top byte
Function Pakreadint() As Integer
Local Dummy As Byte
Local Ival As Integer
Call Paksend(&Hb)
Pakrv = Pakrcv(1)
Call Paksend(3)
Dummy = Pakrcv(0)
Dummy = Pakrcv(0)
Dummy = Pakrcv(0)
Ival = Dummy * 256
Dummy = Pakrcv(0)
Ival = Ival + Dummy
Pakreadint = Ival
End Function


' Reset PAK communications
Sub PakReset
$asm
  CBI Portb,4
  RCall Pakw
  SBI Portb,3
  RCall Pakw
  SBI Portb,4
  RCall Pakw
  CBI Portb,3
  RCall Pakw

$end Asm
End Sub

' Send a byte
Sub PakSend(byval v as Byte)
Loadadr V , X
$asm
  push Pakctr
  push Pakdata
  ldi Pakctr,8
  ld Pakdata,X
Paksendloop:
  lsl Pakdata
  SBI Portb,4
  brcs paks0
  cbi portb,4
Paks0:
  sbi portb,3
      ; Need Slight Delay
  RCall Pakw
  cbi portb,3
  RCall Pakw
  dec Pakctr
  brne paksendloop
  pop Pakdata
  pop Pakctr

$end Asm
End Sub

' Receive a byte
' If Pkwait is 1 then wait for data line
' to go low. If Pkwait is 0 then don't wait
Function Pakrcv(byval Pkwait As Byte) As Byte
Local Pakrcvv As Byte
Pakrcvv = Pkwait
Loadadr Pakrcvv , X

$asm
  push Pakctr
  push Pakdata
  ldi Pakctr,8
  ld Pakdata,X
  cbi ddrb,4           ; Change Ddr
  sbrs Pakdata,0
  rjmp Pakrvcnow
Pakrwloop:
  sbic pinb,4
  rjmp Pakrwloop
Pakrvcnow:
  lsl Pakdata
  sbic pinb,4
  ori Pakdata,1
  sbi portb,3
    ; Small Delay
  RCall Pakw
  cbi portb,3
  RCall Pakw
  dec Pakctr
  brne Pakrvcnow
  sbi ddrb,4
  st X,Pakdata
  pop Pakdata
  pop Pakctr
$end Asm
Pakrcv = Pakrcvv
End Function

